from guichet.models import (
    Ticket,
    Registration,
    Option,
    Info_Sup_Recv,
)

import django_filters as filters
from django import forms
from functools import reduce


def pending_info_sup(queryset, name, value):
    if not value:
        return queryset
    return queryset.filter(info_sup_received__state=Info_Sup_Recv.STATE_PENDING).distinct()


def opposed_reason_filter(queryset, name, value):
    print(value)
    if not value:
        return queryset
    return queryset.filter(**{name: value, "opposed": True})


def name_search(queryset, name, value):
    value = value.strip()
    if not value:
        return queryset
    if " " in value:
        n1, n2 = value.split(" ", 1)
        q1 = queryset.filter(owner__is_anonyme=False, owner__first_name__icontains=n1, owner__last_name__icontains=n2)
        q2 = queryset.filter(owner__is_anonyme=False, owner__first_name__icontains=n2, owner__last_name__icontains=n1)
        q3 = queryset.filter(payee__icontains=value)
        res = q1 | q2 | q3
    elif "@" in value:
        res = queryset.filter(owner__is_anonyme=False, owner__email__icontains=value)
    else:
        q1 = queryset.filter(owner__is_anonyme=False, owner__first_name__icontains=value)
        q2 = queryset.filter(owner__is_anonyme=False, owner__last_name__icontains=value)
        q3 = queryset.filter(owner__is_anonyme=False, owner__login_inp_net__icontains=value)
        q4 = queryset.filter(payee__icontains=value)
        res = q1 | q2 | q3 | q4
    return res.distinct()


def option_search(queryset, name, value):
    """
    Effectue un tri intelligeant sur les options
     - On ne prend que les reservations dont le ticket possède un des options sélectionnées
     - La réservation doit avoir toutes les options simples sélectionnés de son billet
     - La réservation doit avoir une des options groupés par groupe d'options groupés sélectionnés de son billet
    """
    if not value:
        return queryset
    tickets = Ticket.objects.filter(options__in=value).distinct()
    options = Option.objects.filter(id__in=[v.id for v in value])
    qss = []
    for ticket in tickets:
        opt = options.filter(ticket=ticket)
        opt_simple = opt.filter(group__isnull=True)
        opt_group = opt.filter(group__isnull=False)
        qs = queryset.filter(ticket=ticket)
        for o in opt_simple:
            qs = qs.filter(options__in=[o])
        for group_n in opt_group.values_list("group", flat=True).distinct():
            qs = qs.filter(options__in=opt_group.filter(group=group_n))
        qss.append(qs)
    return reduce((lambda q1, q2: q1 | q2), qss)


class RegistrationUnpaidFilter(filters.FilterSet):
    name = filters.CharFilter(label="Nom du client", method=name_search)
    ticket = filters.ModelMultipleChoiceFilter(label="Billet")
    info_state = filters.ChoiceFilter(choices=Registration.STATE_INFO, label="État des informations supplémentaires")
    pending_info = filters.BooleanFilter(
        label="Informations supplémentaires en attente de validation",
        method=pending_info_sup,
        widget=forms.CheckboxInput(),
    )
    option = filters.ModelMultipleChoiceFilter(label="Options", method=option_search)

    class Meta:
        model = Registration
        fields = []

    def __init__(self, *args, **kwargs):
        self.event = kwargs.pop("event")
        super().__init__(*args, **kwargs)

        self.filters["ticket"].extra.update(
            {
                "queryset": Ticket.objects.filter(event=self.event),
            }
        )
        self.filters["ticket"].field.label_from_instance = lambda t: t.name

        self.filters["option"].extra.update(
            {
                "queryset": Option.objects.filter(ticket__event=self.event),
            }
        )
        self.filters["option"].field.label_from_instance = lambda o: o.search_name()


class RegistrationPaidFilter(filters.FilterSet):
    name = filters.CharFilter(label="Nom du client", method=name_search)
    ticket = filters.ModelMultipleChoiceFilter(label="Billet")
    payment_method = filters.ChoiceFilter(choices=Registration.PAYMENT_METHODS, label="Méthode de paiement")
    info_state = filters.ChoiceFilter(choices=Registration.STATE_INFO, label="État des informations supplémentaires")
    pending_info = filters.BooleanFilter(
        label="Informations supplémentaires en attente de validation",
        method=pending_info_sup,
        widget=forms.CheckboxInput(),
    )
    option = filters.ModelMultipleChoiceFilter(label="Options", method=option_search)
    opposed = filters.BooleanFilter(label="En opposition")
    reason_opposed = filters.ChoiceFilter(
        choices=Registration.OPPOSED_REASON, label="Raison de mise en opposition", method=opposed_reason_filter
    )
    used = filters.BooleanFilter(label="Place utilisée")

    class Meta:
        model = Registration
        fields = []

    def __init__(self, *args, **kwargs):
        self.event = kwargs.pop("event")
        super().__init__(*args, **kwargs)

        self.filters["ticket"].extra.update(
            {
                "queryset": Ticket.objects.filter(event=self.event),
            }
        )
        self.filters["ticket"].field.label_from_instance = lambda t: t.name

        self.filters["option"].extra.update(
            {
                "queryset": Option.objects.filter(ticket__event=self.event),
            }
        )
        self.filters["option"].field.label_from_instance = lambda o: o.search_name()
